const { fs } = require('./../bootstrap/modules');
const { getAppName } = require('./../helpers/manager');
const commander = require('@flk/commander');

commander
    .command('nd, new:middleware <middlewares...>', 'Create new middelware.')
    .option('-m, --module', 'Creates the component inside the given middleware name.')
    .option('--app', 'Define the application name that will contain the installed middleware, default to baseApp value in config.json settings.')
    .then((command) => {
        for (let middlewareName of command.middlewares) {
            createMiddleware(middlewareName, command);
        }

        die();
    });

function createMiddleware(middlewareName, command) {
    let appName = getAppName(command);

    let moduleName = command.options.module;

    let middlewarePath = ROOT + (moduleName ? `/src/${appName}/modules/${moduleName}/middleware/${middlewareName}.js` : `/src/${appName}/modules/middleware/${middlewareName}.js`);

    updatePackage(moduleName, appName);

    if (fs.exists(middlewarePath)) {
        die(`${cli.redBright(middlewareName)} middleware already exists!`);
    }

    echo(`${cli.yellow('Creating')} ${cli.green(middlewareName)} middleware...`);

    // first we will copy the middleware
    fs.copy(FRAMEWORK_RESOURCES_PATH + '/middleware.js', middlewarePath);

    let middlewareFileContent = fs.get(middlewarePath);

    let middlewareClass = middlewareName.toStudlyCase() + 'Middleware';

    middlewareFileContent = middlewareFileContent.replaceAll('MiddlewareClass', middlewareClass)
        .replaceAll('middlewareName', middlewareName);

    fs.put(middlewarePath, middlewareFileContent);

    echo(`${cli.cyan(middlewareName)} ${cli.green('middleware has been created successfully.')}`);
}

function updatePackage(moduleName, appName) {
    let packagePath = ROOT + `/src/${appName}/package.json`;

    let $package = fs.getJson(packagePath);

    if (moduleName && $package.modules.includes(moduleName)) {
        $package.modules.push(moduleName);
    } else if (!$package.modules.includes('middleware')) {
        $package.modules.push('middleware');
    }

    fs.putJson(packagePath, $package);
}

module.exports = createMiddleware;