const createService = require('./new-service');
const createComponent = require('./new-component');
const { fs } = require('./../bootstrap/modules');
const { getAppName } = require('./../helpers/manager');
const commander = require('@flk/commander');

commander
    .command('nm, new:module <module>', 'Create new module.')
    .option('--app', 'Define the application name that will contain the installed module, default to baseApp value in config.json settings.')
    .option('-m, --module', 'Creates the component inside the given module name.')
    .option('-r, --route', 'Create route for component and service.')
    .then(command => {
        createModule(command);
        die();
    });

function createModule(command) {
    let moduleName = command.module;
    let appName = getAppName(command);

    let modulePath = ROOT + `/src/${appName}/modules/${moduleName}`;

    if (fs.exists(modulePath)) {
        die(`${cli.redBright(moduleName)} module already exists!`);
    }

    echo(`${cli.yellow('Creating')} ${cli.green(moduleName)} module...`);

    if (!command.service) {
        command.service = moduleName;
    }

    command.components = [moduleName];
    command.options.module = moduleName;

    let componentOptions = createComponent(command.components[0], command);

    let serviceOptions = createService(command);

    // create locales directory

    fs.makeDirectory(modulePath + '/locales');

    for (let locale of appConfig.locales) {
        let localeContent = `
Locale.extend('${locale}', {
    '${componentOptions.title}': '${componentOptions.selector.replace('-page', '').split('-').join(' ').capitalize()}',
});`.trim();
        fs.put(modulePath + `/locales/${locale}.js`, localeContent);
    }

    echo(`${cli.cyan(moduleName)} ${cli.green('module has been created successfully.')}`);

    return {
        componentOptions,
        serviceOptions,
    }
}

module.exports = createModule;