const { fs } = require('./../bootstrap/modules');
const { getAppName } = require('./../helpers/manager');

const commander = require('@flk/commander');

commander
    .command('ns, new:service <service>', 'Create new service.')
    .option('--app', 'Define the application name that will contain the installed service, default to baseApp value in config.json settings.')
    .option('-m, --module', 'Creates the component inside the given module name.')
    .option('-r, --route', 'Endpoint service route.')
    .option('-a, --class', 'Service class name, default to service name in studly case.')
    .option('-a, --alias', 'Service alias name, default to service name in camel case.')
    .then(command => {
        createService(command);
        die();
    });

function createService(command) {
    let appName = getAppName(command);

    let serviceName = command.service;

    let serviceSegments = serviceName.split('/');

    let service = command.options.class || Array.end(serviceSegments).toStudlyCase() + 'Service';

    let moduleName = command.options.module;

    let servicePath = ROOT + (moduleName ? `/src/${appName}/modules/${moduleName}/services/${serviceName}-service.js` : `/src/${appName}/modules/services/${serviceName}-service.js`);

    updatePackage(moduleName, appName);

    if (fs.exists(servicePath)) {
        die(`${cli.redBright(serviceName)} service already exists!`);
    }

    echo(`${cli.yellow('Creating')} ${cli.green(serviceName)} service...`);

    // first we will copy the service
    fs.copy(FRAMEWORK_RESOURCES_PATH + '/service.js', servicePath);

    let serviceFileContent = fs.get(servicePath);

    let alias = command.options.alias || service.toCamelCase();

    let route = command.options.route || '/' + serviceName;

    serviceFileContent = serviceFileContent.replace('aliasName', alias)
        .replace('route', route)
        .replaceAll('ServiceName', service);

    fs.put(servicePath, serviceFileContent);

    echo(`${cli.cyan(serviceName)} ${cli.green('service has been created successfully.')}`);

    return {
        class: service,
        alias,
        route,
    };
}

function updatePackage(moduleName, appName) {
    let packagePath = ROOT + `/src/${appName}/package.json`;

    let $package = fs.getJson(packagePath);

    if (moduleName) {
        if (!$package.modules.includes(moduleName)) {
            $package.modules.push(moduleName);
        }
    } else if (!$package.modules.includes('services')) {
        $package.modules.push('services');
    }

    fs.putJson(packagePath, $package);
}

module.exports = createService;