const commander = require('@flk/commander');

commander
    .command('serve [app]', 'Start serving application.')
    .option('-p, --port', 'Set the opening port, default is in the config.json file.')
    .option('-sp, --socket-port', 'Set the hot reloading port, default is ' + process.env.SOCKET_PORT)
    .option('-o, --open', 'Open the application in browser.')
    .option('-b, --browser', 'Open the application in the given browser name.')
    .examples(
        `${cli.cyanBright(`${process.env.CLI_NAME} serve`)} >> ${cli.yellow('Serve default application')}`,
        `${cli.cyanBright(`${process.env.CLI_NAME} serve admin`)} >> ${cli.yellow('Serve admin application')}`,
        `${cli.cyanBright(`${process.env.CLI_NAME} serve -o`)} >> ${cli.yellow('Serve default application and open it in the browser.')}`,
        `${cli.cyanBright(`${process.env.CLI_NAME} serve -o -b=chrome`)} >> ${cli.yellow('Serve default application and open it in google chrome browser.')}`,
    )
    .then(async command => {
        const portIsTaken = require('./../helpers/port-usage');
        function validatePorts() {
            return new Promise(async resolve => {
                process.env.PORT = configApp.port;
                global.ORIGINAL_PORT = process.env.PORT;

                while (await portIsTaken(process.env.PORT)) {
                    process.env.PORT++;
                }

                if (global.ORIGINAL_PORT != process.env.PORT) {
                    global.BASE_URL = global.BASE_URL.replace(global.ORIGINAL_PORT, process.env.PORT);
                    global.SCRIPT_URL = global.SCRIPT_URL.replace(global.ORIGINAL_PORT, process.env.PORT);
                }

                while (await portIsTaken(process.env.SOCKET_PORT)) {
                    process.env.SOCKET_PORT++;
                }

                resolve();
            });
        }

        await validatePorts();

        require(FRAMEWORK_ROOT_PATH + '/server/index')(command);
    });
